import _concatInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/concat";
import _forEachInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/for-each";
import _extends from "@babel/runtime-corejs3/helpers/esm/extends";
import _regeneratorRuntime from "@babel/runtime-corejs3/regenerator";
import _everyInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/every";
import _Promise from "@babel/runtime-corejs3/core-js-stable/promise";
import _mapInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/map";
import _someInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/some";
import _awaitAsyncGenerator from "@babel/runtime-corejs3/helpers/esm/awaitAsyncGenerator";
import _wrapAsyncGenerator from "@babel/runtime-corejs3/helpers/esm/wrapAsyncGenerator";
import { accumulateRouteValues, checkResolved, getComponents, getRouteMatches, getRouteValues, isResolved } from 'found/ResolverUtils';
import isPromise from 'is-promise';
import isEqual from 'lodash/isEqual';
import React from 'react';
import warning from 'warning';
import QuerySubscription from './QuerySubscription';
import ReadyStateRenderer from './ReadyStateRenderer';
import renderElement from './renderElement';

var Resolver = /*#__PURE__*/function () {
  function Resolver(environment) {
    this.environment = environment;
    this.lastQuerySubscriptions = [];
  }

  var _proto = Resolver.prototype;

  _proto.resolveElements = function resolveElements(match) {
    var _this = this;

    return _wrapAsyncGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
      var routeMatches, Components, queries, cacheConfigs, fetchPolicies, routeVariables, querySubscriptions, fetches, earlyComponents, earlyData, fetchedComponents, pendingElements;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              routeMatches = getRouteMatches(match);
              Components = getComponents(routeMatches);
              queries = getRouteValues(routeMatches, function (route) {
                return route.getQuery;
              }, function (route) {
                return route.query;
              });
              cacheConfigs = getRouteValues(routeMatches, function (route) {
                return route.getCacheConfig;
              }, function (route) {
                return route.cacheConfig;
              });
              fetchPolicies = getRouteValues(routeMatches, function (route) {
                return route.getFetchPolicy;
              }, function (route) {
                return route.fetchPolicy;
              });
              process.env.NODE_ENV !== "production" ? warning(!_someInstanceProperty(routeMatches).call(routeMatches, function (_ref) {
                var route = _ref.route;
                return route.dataFrom;
              }), '`dataFrom` on routes no longer has any effect; use `fetchPolicy` instead.') : void 0;
              process.env.NODE_ENV !== "production" ? warning(!_someInstanceProperty(routeMatches).call(routeMatches, function (_ref2) {
                var route = _ref2.route;
                return route.getDataFrom;
              }), '`getDataFrom` on routes no longer has any effect; use `getFetchPolicy` instead.') : void 0;
              routeVariables = _this.getRouteVariables(match, routeMatches);
              querySubscriptions = _this.updateQuerySubscriptions(queries, routeVariables, cacheConfigs, fetchPolicies);
              fetches = _mapInstanceProperty(querySubscriptions).call(querySubscriptions, function (querySubscription) {
                return querySubscription && querySubscription.fetch();
              });

              if (!_someInstanceProperty(Components).call(Components, isPromise)) {
                _context.next = 16;
                break;
              }

              _context.next = 13;
              return _awaitAsyncGenerator(_Promise.all(_mapInstanceProperty(Components).call(Components, checkResolved)));

            case 13:
              _context.t0 = _context.sent;
              _context.next = 17;
              break;

            case 16:
              _context.t0 = Components;

            case 17:
              earlyComponents = _context.t0;
              _context.next = 20;
              return _awaitAsyncGenerator(_Promise.all(_mapInstanceProperty(fetches).call(fetches, checkResolved)));

            case 20:
              earlyData = _context.sent;

              if (!(!_everyInstanceProperty(earlyComponents).call(earlyComponents, isResolved) || !_everyInstanceProperty(earlyData).call(earlyData, isResolved))) {
                _context.next = 32;
                break;
              }

              pendingElements = _this.createElements(routeMatches, earlyComponents, querySubscriptions, false);
              _context.next = 25;
              return _everyInstanceProperty(pendingElements).call(pendingElements, function (element) {
                return element !== undefined;
              }) ? pendingElements : undefined;

            case 25:
              _context.next = 27;
              return _awaitAsyncGenerator(_Promise.all(Components));

            case 27:
              fetchedComponents = _context.sent;
              _context.next = 30;
              return _awaitAsyncGenerator(_Promise.all(fetches));

            case 30:
              _context.next = 33;
              break;

            case 32:
              fetchedComponents = earlyComponents;

            case 33:
              _context.next = 35;
              return _this.createElements(routeMatches, fetchedComponents, querySubscriptions, true);

            case 35:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }))();
  };

  _proto.getRouteVariables = function getRouteVariables(match, routeMatches) {
    return accumulateRouteValues(routeMatches, match.routeIndices, function (variables, routeMatch) {
      var route = routeMatch.route,
          routeParams = routeMatch.routeParams; // We need to always run this to make sure we don't miss route params.

      var nextVariables = _extends({}, variables, {}, routeParams);

      if (route.prepareVariables) {
        nextVariables = route.prepareVariables(nextVariables, routeMatch);
      }

      return nextVariables;
    }, null);
  };

  _proto.updateQuerySubscriptions = function updateQuerySubscriptions(queries, routeVariables, cacheConfigs, fetchPolicies) {
    var _this2 = this,
        _context2,
        _context3;

    var querySubscriptions = _mapInstanceProperty(queries).call(queries, function (query, i) {
      if (!query) {
        return null;
      }

      var variables = routeVariables[i];
      var lastQuerySubscription = _this2.lastQuerySubscriptions[i]; // Match the logic in <QueryRenderer> for not refetching.

      if (lastQuerySubscription && lastQuerySubscription.query === query && isEqual(lastQuerySubscription.variables, variables)) {
        _this2.lastQuerySubscriptions[i] = null;
        return lastQuerySubscription;
      }

      return new QuerySubscription({
        environment: _this2.environment,
        query: query,
        variables: variables,
        cacheConfig: cacheConfigs[i],
        fetchPolicy: fetchPolicies[i]
      });
    });

    _forEachInstanceProperty(_context2 = this.lastQuerySubscriptions).call(_context2, function (querySubscription) {
      if (querySubscription) {
        querySubscription.dispose();
      }
    });

    this.lastQuerySubscriptions = _concatInstanceProperty(_context3 = []).call(_context3, querySubscriptions);
    return querySubscriptions;
  };

  _proto.createElements = function createElements(routeMatches, Components, querySubscriptions, fetched) {
    return _mapInstanceProperty(routeMatches).call(routeMatches, function (match, i) {
      var route = match.route,
          router = match.router;
      var Component = Components[i];
      var querySubscription = querySubscriptions[i];
      var isComponentResolved = isResolved(Component); // Handle non-Relay routes.

      if (!querySubscription) {
        if (route.render) {
          return route.render({
            match: match,
            Component: isComponentResolved ? Component : null,
            props: {
              match: match,
              router: router
            }
          });
        }

        if (!isComponentResolved) {
          return undefined;
        }

        return Component ? /*#__PURE__*/React.createElement(Component, {
          match: match,
          router: router
        }) : null;
      }

      var resolvedComponent = isComponentResolved ? Component : null;
      var hasComponent = Component != null;
      var element = renderElement({
        match: match,
        Component: resolvedComponent,
        isComponentResolved: isComponentResolved,
        hasComponent: hasComponent,
        querySubscription: querySubscription,
        resolving: true
      });

      if (!element) {
        return element;
      }

      return function (routeChildren) {
        return /*#__PURE__*/React.createElement(ReadyStateRenderer, {
          match: match,
          Component: resolvedComponent,
          isComponentResolved: isComponentResolved,
          hasComponent: hasComponent,
          element: element,
          routeChildren: routeChildren,
          querySubscription: querySubscription,
          fetched: fetched
        });
      };
    });
  };

  return Resolver;
}();

export { Resolver as default };